#!/usr/bin/env ruby

#
# WPScan - WordPress Security Scanner
# Copyright (C) 2011  Ryan Dewhurst AKA ethicalhack3r
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ryandewhurst at gmail
#

class Validate

  attr_accessor :url, :username, :wordlist

  def initialize(url,username,wordlist)
    @url = url
    @username = username
    @wordlist = wordlist
  end

  # Ensures the url is in the correct format

  def url

    @url = add_slash!(@url)
    @url = add_protocol!(@url)
    
    begin
      @url = URI.parse(@url)
    rescue => e
      puts
      puts '[ERROR] URL not in correct format... ' + e.inspect
      puts @url.inspect
      exit(1)      
    end

  end

  # Add a leading slash to a url
  
  def add_slash!(url)
    if url !~ /\/$/
      url = url + '/'
    else
      url = url
    end 
  end

  # Remove a leading slash from a url 

  def remove_slash!(url)
    url = url.to_s
    if url =~ /\/$/
      url = url.gsub!(/\/$/,'')
    else
      url = url
    end
  end

  # Add protocol

  def add_protocol!(url)
    if url !~ /^https?:/
      url = 'http://' + url
    else
      url = url
    end
  end

  # does the wordlist file exist?
  
  def wordlist
    if @wordlist != ''
      if !File.exists? @wordlist
        return false
      end
    end
  end

  # Checks if the remote website is up.
  #
  # checks to see if the remote website,
  # is responding to our requests by,
  # first trying a DNS request, if that fails,
  # then try a reverse DNS lookup, if that,
  # failes, we assume the remote host is down.

  def website_up?

    up = true

    response = Typhoeus::Request.get(@url.to_s)
 
    if response.code == 0
      up = false
    end

    return up

  end

  # check if the remote website is
  # actually running wordpress.

  def is_wordpress?

    wordpress = false

    response = Typhoeus::Request.get(@url.to_s + 'wp-login.php', :follow_location => true, :max_redirects => 2)
    response2 = Typhoeus::Request.get(@url.to_s + 'xmlrpc.php', :follow_location => true, :max_redirects => 2)

    if response.body =~ %r{WordPress}i
      wordpress = true
    end

    if response2.body =~ %r{XML-RPC server accepts POST requests only}i
      wordpress = true
    end

    wordpress
 
  end

  # see if the remote url returns 30x redirect

  def is_redirect?
  
    redirect = false

    response = Typhoeus::Request.get(@url.to_s)

    if response.code == 301 || response.code == 302
      redirect = response.headers_hash['location']
    end

  end

end
