#!/usr/bin/env ruby

#
# WPScan - WordPress Security Scanner
# Copyright (C) 2011  Ryan Dewhurst AKA ethicalhack3r
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ryandewhurst at gmail
#

$: << '.'

begin
  # standard libs
  require 'rubygems'
  require 'getoptlong'
  require 'uri'
  require 'time'
  require 'resolv'
  require 'xmlrpc/client'
  require 'digest/md5'
  require 'readline'
  require 'base64'
  require 'cgi'
  # custom libs
  require 'lib/discover'
  require 'lib/bruter'
  require 'lib/validate'
  require 'lib/msfrpc_client'
  require 'lib/exploit'
  require 'lib/update'
  require 'tools/generate_plugin_list'
  # third party libs
  require 'typhoeus'
  require 'xmlsimple'
rescue LoadError => e
  puts "[ERROR] Install missing ruby gem. Please see README file or http://code.google.com/p/wpscan/"
  puts e.inspect
  exit(1)
end

# Declare needed variables

url = ''
enum_version = false
enum_plugins = false
enum_timthumb = false
enum_users = false
generate_plugin_list = 0
username = ''
wordlist = ''
password = ''
threads = 20
verbose = false
@version = "1.1"

# our 1337 banner

def banner()
  puts '____________________________________________________'
  puts " __          _______   _____                  "
  puts " \\ \\        / /  __ \\ / ____|                 "
  puts "  \\ \\  /\\  / /| |__) | (___   ___  __ _ _ __  "
  puts "   \\ \\/  \\/ / |  ___/ \\___ \\ / __|/ _` | '_ \\ "
  puts "    \\  /\\  /  | |     ____) | (__| (_| | | | |"
  puts "     \\/  \\/   |_|    |_____/ \\___|\\__,_|_| |_|" + " v" + @version
  puts
  puts "  WordPress Security Scanner by ethicalhack3r.co.uk"
  puts " Sponsored by the RandomStorm Open Source Initiative"
  puts '_____________________________________________________'
  puts
end

# wpscan usage

def usage()
  script_name = $0 
  puts
  puts "-h for further help."
  puts 
  puts "Examples:"
  puts
  puts "Do 'non-intrusive' checks..."
  puts "ruby " + script_name + " --url www.example.com"
  puts
  puts "Do wordlist password brute force on enumerated users using 50 threads..."
  puts "ruby " + script_name + " --url www.example.com --wordlist darkc0de.lst --threads 50"
  puts
  puts "Do wordlist password brute force on the 'admin' username only..."
  puts "ruby " + script_name + " --url www.example.com --wordlist darkc0de.lst --username admin"
  puts
  puts "Generate a new 'most popular' plugin list, up to 150 pages..."
  puts "ruby " + script_name + " --generate_plugin_list 150"
  puts
  puts "Enumerate instaled plugins..."
  puts "ruby " + script_name + " --url www.example.com --enumerate p"
  puts 
  puts 'See README for further information.'
end

# command help

def help()
  puts "Help:"
  puts
  puts "--url				The WordPress URL/domain to scan."
  puts "--enumerate			Enumeration."
  puts "u				users"
  puts "v				version"
  puts "p				plugins"
  puts "t				timthumb"
  puts "--wordlist			Supply a wordlist for the password bruter and do the brute."
  puts "--threads			The number of threads to use when multi-threading requests."
  puts "--username			Only brute force the supplied username."
  puts "--generate_plugin_list		Generate a new data/plugins.txt file. (supply number of *pages* to parse)"
  puts "-h				This help screen."
  puts "-v				Verbose output."
  puts
end

# Output banner

banner()
puts

# Update to the latest SVN revision
# This should be done as early in the code as possible.

if Update.new.update
  exit
end

# do we have any input?!

if ARGV.length == 0
  usage()
  exit
end

# Deal with user supplied input

opts = GetoptLong.new(
  ["--url", "-u", GetoptLong::REQUIRED_ARGUMENT],
  ["--enumerate", "-e", GetoptLong::OPTIONAL_ARGUMENT],
  ["--generate_plugin_list", GetoptLong::OPTIONAL_ARGUMENT],
  ["--username", "-U", GetoptLong::REQUIRED_ARGUMENT],
  ["--wordlist", "-w", GetoptLong::REQUIRED_ARGUMENT],
  ["--threads", "-t",GetoptLong::REQUIRED_ARGUMENT],
  ["-h", GetoptLong::NO_ARGUMENT],
  ["-v", GetoptLong::NO_ARGUMENT] 
) 

begin

  opts.each do |opt, arg|

    case opt

      when '--url' || '-u'
        url = arg
      when '--enumerate' || '-e'
	if arg == "v"
          enum_version = true
        elsif arg == "p"
          enum_plugins = true
        elsif arg == "t"
          enum_timthumb = true
        elsif arg == "u"
          enum_users = true
        elsif arg == ""
          enum_version = true
          enum_plugins = true
          enum_timthumb = true
          enum_users = true
        else
          puts "[ERROR] Unknown enumerate option. v = version, p = plugins, t = timthumb, u = users"
          exit(1)
        end
      when '--generate_plugin_list'
        if arg == ""
          puts "Number of pages not supplied, defaulting to 150 pages..."
          puts
          generate_plugin_list = 150
        else
          generate_plugin_list = arg
        end
      when '--username' || '-U'
        username = arg
      when '--wordlist' || '-w'
        wordlist = arg
      when '--threads' || '-t'
          threads = arg.to_i
      when '-h'
          help()
          exit(1)
      when '-v' 
        verbose = true 
      end

   end

rescue
  usage()
  help()
  exit
end

# Generates a new plugin list for plugin enumeration

unless generate_plugin_list == 0
  puts "[+] Generating new most popular plugin list"
  puts
  Generate_Plugin_List.new(generate_plugin_list,threads,verbose).save_file
  exit if url == ""
end

# Validate user input...

validate = Validate.new(url,username,wordlist)
url = validate.url

# Wordlist exists?

if validate.wordlist == false
  puts "[ERROR] The file \"" + wordlist.to_s + "\" does not exist."
  exit
end

# Remote website up?

if validate.website_up? == false
  puts "[ERROR] The WordPress URL supplied \"" + url.to_s  + "\" seems to be down."
  exit
end

# Remote website is a redirect?

redirect = validate.is_redirect?
if redirect
  puts "[ERROR] The remote host tried to redirect us to " + redirect + "."
  puts "Use that URL if it is the correct one."
  exit
end

# Remote website is wordpress?

if validate.is_wordpress? == false
  puts "[ERROR] The remote website is up, but does not seem to be running WordPress."
  exit(1)
end

# Output runtime data

puts '| URL: ' + url.to_s
puts '| Started on ' + Time.now.asctime
puts 

# Create the discover object

discover = Discover.new(url, threads, verbose)

# Can we identify the theme name?

theme_name = discover.theme_name
unless theme_name == false or theme_name == '' or theme_name.nil?
  puts "[!] The WordPress theme in use is called \"" + theme_name.to_s + "\"."
end

# Is the readme.html file there?

if discover.readme_exists?
  puts "[!] The WordPress \"" + url.to_s  + "readme.html\" file exists."
end

# Full Path Disclosure (FPD)?

if discover.full_path_disclosure?
  puts "[!] Full Path Disclosure (FPD) in \"" + url.to_s  + "wp-includes/rss-functions.php\"."
end

# If the html generator meta tag is in the source, extract the version...

gen_version = discover.generator_version

unless gen_version.nil?
  puts '[!] WordPress version ' + gen_version.to_s  + ' identified from meta generator.'
  version_discovered = "generator"
else # otherwise do 'advanced fingerprinting'
  advanced_version = discover.advanced_version_detection
  if advanced_version
    puts '[!] WordPress version/s "' + advanced_version  + '" identified from advanced fingerprinting.'
    version_discovered = "advanced"
  end
end

# Are there any vulnerabilities associated with this version?

if version_discovered == "advanced" || version_discovered == "generator"

  if version_discovered == "generator"
    wp_vulnerabilities = discover.wp_vulnerabilities(gen_version)
  else
    wp_vulnerabilities = discover.wp_vulnerabilities(advanced_version)
  end

  unless wp_vulnerabilities.nil? or wp_vulnerabilities == ''
    count = 0
    puts
    puts '[+] We have identified ' + wp_vulnerabilities.size.to_s  + ' vulnerabilities from the version number: '
    wp_vulnerabilities.each do |vulnerability|
      puts
      puts '* Title: ' + vulnerability[1][count]['title'][0]
      puts '* Reference: ' + vulnerability[1][count]['reference'][0]
      count += 1
    end
  end

end

# If enum_plugins = true, enumerate the installed plugins

if enum_plugins

  puts
  puts "[+] Enumerating installed plugins..."
  puts

  plugins = discover.plugins
  unless plugins.empty?

    puts
    puts
    puts "[+] We found " + plugins.size.to_s  + " plugins:"
    puts  

    plugins.each do |plugin|
      puts "Name: " + plugin
      puts "Location: " + url.to_s + "wp-content/plugins/" + plugin + "/"
      # Is directory listing enabled?
      if discover.directory_listing?(plugin)
        puts "Directory listing enabled? Yes."
      else
        puts "Directory listing enabled? No."
      end
      puts
    end

    # are there any vulnerabilities associated with the found plugins?

    plugin_vulns = discover.plugin_vulnerabilities(plugins)
    unless plugin_vulns.nil? or plugin_vulns == '' or plugin_vulns.empty?

      plugin_vulns = plugin_vulns[0]['vulnerability']

      puts '[+] There were ' + plugin_vulns.size.to_s  + ' vulnerabilities identified from the plugin names:'

      # output the found vulns
      plugin_vulns.each do |vulnerability|

        #puts vulnerability['title'][0].inspect

        title = vulnerability['title'][0]
        reference = vulnerability['reference'][0]
        type = vulnerability['type'][0]
        vulnerability['uri'] == nil ? "" : uri = vulnerability['uri'][0] # uri
        vulnerability['postdata'] == nil ? "" : postdata = CGI.unescapeHTML(vulnerability['postdata'][0]) # postdata

        puts
        puts '[!] ' + title.to_s
        puts '* Reference: ' + reference.to_s

        # This has been commented out as MSF are moving from
        # XML-RPC to MessagePack.
        # I need to get to grips with the new way of communicating
        # with MSF and implement new code. 

        # check if vuln is exploitable
        #Exploit.new(url, type, uri, postdata.to_s)

      end
    end

    # Is there a error_log file in te plugins dir?
    error_logs = discover.error_log(plugins)
    unless error_logs.empty? or error_logs.nil?
      error_logs.each do |error_log|
        puts
        puts "[!] A WordPress error_log file has been found: " + error_log.to_s
      end
    end

  else
    puts
    puts "No plugins found :("
    puts
  end

end

# If enum_timthumb = true, try to find timthumb files

if enum_timthumb

  puts
  puts "[+] Enumerating timthumb files..."
  puts
  
  timthumb = discover.timthumb(theme_name)
  unless timthumb.empty?

    puts
    puts "[+] We found " + timthumb.size.to_s  + " timthumb files:"
    puts
    puts timthumb.flatten
    puts
    puts "* Reference: http://wwww.exploit-db.com/exploits/17602/"

  else
    puts
    puts "No timthumb files found :("
  end

end
# If we haven't been supplied a username, enumerate them...

if username == '' and wordlist != '' or enum_users == true

  puts
  puts '[+] Enumerating usernames...'

  usernames = discover.usernames

  if usernames.empty?
    puts 
    puts "We did not enumerate any usernames :("
    puts "Try supplying your own username with the --username option"
    puts
    exit(1) 
  else 
    puts 
    puts 'We found the following ' + usernames.length.to_s + ' username/s:'
    puts
    puts usernames
  end

else 

  usernames = [username]

end

# Start the brute forcer

if wordlist != ''

  # Is there is a login-lockdown plugin enabled?
  if discover.login_lockdown?
    puts
    puts "[ERROR] login-lockdown plugin enabled. It will record the IP and timestamp of every failed login. Not a good idea for brute forcing!"
    exit(1)
  end

  puts
  puts '[+] Starting the password brute forcer'
  puts
  Bruter.new(url,usernames,wordlist,threads,verbose).brute
end

puts
puts '[+] Finished at ' + Time.now.asctime
exit() # must exit!
